using System.IdentityModel.Tokens.Jwt;
using System.Security.Principal;
using System.Text;
using Jose;
using Microsoft.IdentityModel.Tokens;

// ReSharper disable RedundantExplicitArrayCreation

namespace FitConnect.Encryption;

public class JoseEncryptor : IEncryptor {
    private const JweEncryption Encryption = JweEncryption.A256GCM;
    private const JweCompression Compression = JweCompression.DEF;
    private const JweAlgorithm Algorithm = JweAlgorithm.RSA_OAEP_256;
    private const SerializationMode SerializationMode = Jose.SerializationMode.Compact;

    private const string ErrorMessage =
        "On macOS add DYLD_LIBRARY_PATH to your environment variables. Look at the README for more information.";


    public (string plainText, byte[] plainBytes, byte[] tag)
        Decrypt(string key, string cipher) {
        try {
            var jwk = Jwk.FromJson(key, new JsonMapper());
            return Decrypt(jwk, cipher);
        }
        catch (PlatformNotSupportedException e) {
            throw new PlatformNotSupportedException(ErrorMessage, e);
        }
    }

    public string Encrypt(string key, string plain) {
        // ti1Z0KkG1uAXvd2XnWb5wWu8slGylvsvz3nOSe7yuAc
        try {
            var jwk = Jwk.FromJson(key, new JsonMapper());
            return JWE.Encrypt(plain,
                new JweRecipient[] { new(Algorithm, jwk) },
                Encryption,
                compression: Compression,
                mode: SerializationMode,
                extraProtectedHeaders: new Dictionary<string, object> {
                    { "kid", jwk.KeyId },
                    { "cty", "application/json" }
                }
            );
        }
        catch (PlatformNotSupportedException e) {
            throw new PlatformNotSupportedException(ErrorMessage, e);
        }
    }

    public string Encrypt(string key, byte[] plain) {
        try {
            var jwk = Jwk.FromJson(key, new JsonMapper());
            return JWE.EncryptBytes(plain,
                new JweRecipient[] { new(Algorithm, jwk) },
                Encryption,
                compression: Compression,
                mode: SerializationMode,
                extraProtectedHeaders: new Dictionary<string, object> {
                    { "kid", jwk.KeyId },
                    { "cty", "application/json" }
                }
            );
        }
        catch (PlatformNotSupportedException e) {
            throw new PlatformNotSupportedException(ErrorMessage, e);
        }
    }

    private (string plainText, byte[] plainBytes, byte[] tag) Decrypt(Jwk key, string payload) {
        var result = JWE.Decrypt(payload, key, Algorithm, Encryption);

        return (result.Plaintext, result.PlaintextBytes, result.AuthTag);
    }


    /// <summary>
    /// 
    /// </summary>
    /// <param name="key"></param>
    /// <param name="payload"></param>
    /// <returns></returns>
    public static bool CheckSignature(string token, string key) {
        var tokenHandler = new JwtSecurityTokenHandler();

        var validationParameters = new TokenValidationParameters {
            IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(key)),
            ValidAlgorithms = new[] { SecurityAlgorithms.HmacSha512 },
        };

        var principal =
            tokenHandler.ValidateToken(token, validationParameters, out var validatedToken);
        return true;
    }
}
